# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "monitor log-analytics workspace update",
)
class Update(AAZCommand):
    """Update a workspace instance.

    :example: Update a workspace instance.
        az monitor log-analytics workspace update --resource-group myresourcegroup --retention-time 30 --workspace-name myworkspace

    :example: Update the defaultDataCollectionRuleResourceId of the workspace
        az monitor log-analytics workspace update --resource-group myresourcegroup --workspace-name myworkspace --data-collection-rule "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/dataCollectionRules/{dcrName}".
    """

    _aaz_info = {
        "version": "2025-02-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.operationalinsights/workspaces/{}", "2025-02-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.workspace_name = AAZStrArg(
            options=["-n", "--name", "--workspace-name"],
            help="Name of the Log Analytics Workspace.",
            required=True,
            id_part="name",
            fmt=AAZStrArgFormat(
                pattern="^[A-Za-z0-9][A-Za-z0-9-]+[A-Za-z0-9]$",
                max_length=63,
                min_length=4,
            ),
        )

        # define Arg Group "Identity"

        _args_schema = cls._args_schema
        _args_schema.identity_type = AAZStrArg(
            options=["--type", "--identity-type"],
            arg_group="Identity",
            help="Type of managed service identity.",
            enum={"None": "None", "SystemAssigned": "SystemAssigned", "UserAssigned": "UserAssigned"},
        )
        _args_schema.user_assigned = AAZDictArg(
            options=["--user-assigned"],
            arg_group="Identity",
            help="The list of user identities associated with the resource. The user identity dictionary key references will be ARM resource ids in the form: '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.",
            nullable=True,
        )

        user_assigned = cls._args_schema.user_assigned
        user_assigned.Element = AAZObjectArg(
            nullable=True,
            blank={},
        )

        # define Arg Group "Parameters"

        _args_schema = cls._args_schema
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            arg_group="Parameters",
            help="Resource tags.",
            nullable=True,
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg(
            nullable=True,
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.data_collection_rule = AAZStrArg(
            options=["--data-collection-rule"],
            arg_group="Properties",
            help="The resource ID of the default Data Collection Rule to use for this workspace. Expected format is - /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/dataCollectionRules/{dcrName}.",
            nullable=True,
        )
        _args_schema.ingestion_access = AAZStrArg(
            options=["--ingestion-access"],
            arg_group="Properties",
            help="The public network access type to access workspace ingestion.",
            nullable=True,
            enum={"Disabled": "Disabled", "Enabled": "Enabled"},
        )
        _args_schema.query_access = AAZStrArg(
            options=["--query-access"],
            arg_group="Properties",
            help="The public network access type to access workspace query.",
            nullable=True,
            enum={"Disabled": "Disabled", "Enabled": "Enabled"},
        )
        _args_schema.retention_time = AAZIntArg(
            options=["--retention-time"],
            arg_group="Properties",
            help="The workspace data retention in days. Allowed values are per pricing plan. See pricing tiers documentation for details.",
            nullable=True,
        )
        _args_schema.quota = AAZFloatArg(
            options=["--quota"],
            arg_group="Properties",
            help="The workspace daily quota for ingestion in gigabytes. The minimum value is 0.023 and default is -1 which means unlimited.",
            nullable=True,
        )

        # define Arg Group "Replication"

        _args_schema = cls._args_schema
        _args_schema.replication_enabled = AAZBoolArg(
            options=["--replication-enabled"],
            arg_group="Replication",
            help="Specifies whether the replication is enabled or not. When true, workspace configuration and data is replicated to the specified location. If replication is been enabled, location must be provided.",
            nullable=True,
        )

        # define Arg Group "Sku"

        _args_schema = cls._args_schema
        _args_schema.capacity_reservation_level = AAZIntArg(
            options=["--level", "--capacity-reservation-level"],
            arg_group="Sku",
            help="The capacity reservation level for this workspace, when CapacityReservation sku is selected. The maximum value is 1000 and must be in multiples of 100. If you want to increase the limit, please contact LAIngestionRate@microsoft.com.",
            nullable=True,
            enum={"100": 100, "1000": 1000, "10000": 10000, "200": 200, "2000": 2000, "25000": 25000, "300": 300, "400": 400, "500": 500, "5000": 5000, "50000": 50000},
        )
        _args_schema.sku_name = AAZStrArg(
            options=["--sku", "--sku-name"],
            arg_group="Sku",
            help="The name of the SKU.",
            enum={"CapacityReservation": "CapacityReservation", "Free": "Free", "LACluster": "LACluster", "PerGB2018": "PerGB2018", "PerNode": "PerNode", "Premium": "Premium", "Standalone": "Standalone", "Standard": "Standard"},
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.WorkspacesGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        yield self.WorkspacesCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class WorkspacesGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/Microsoft.OperationalInsights/workspaces/{workspaceName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "workspaceName", self.ctx.args.workspace_name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-02-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_workspace_read(cls._schema_on_200)

            return cls._schema_on_200

    class WorkspacesCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/Microsoft.OperationalInsights/workspaces/{workspaceName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "workspaceName", self.ctx.args.workspace_name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-02-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _UpdateHelper._build_schema_workspace_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("identity", AAZIdentityObjectType)
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            identity = _builder.get(".identity")
            if identity is not None:
                identity.set_prop("type", AAZStrType, ".identity_type", typ_kwargs={"flags": {"required": True}})
                identity.set_prop("userAssignedIdentities", AAZDictType, ".user_assigned")

            user_assigned_identities = _builder.get(".identity.userAssignedIdentities")
            if user_assigned_identities is not None:
                user_assigned_identities.set_elements(AAZObjectType, ".")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("defaultDataCollectionRuleResourceId", AAZStrType, ".data_collection_rule")
                properties.set_prop("publicNetworkAccessForIngestion", AAZStrType, ".ingestion_access")
                properties.set_prop("publicNetworkAccessForQuery", AAZStrType, ".query_access")
                properties.set_prop("replication", AAZObjectType)
                properties.set_prop("retentionInDays", AAZIntType, ".retention_time", typ_kwargs={"nullable": True})
                properties.set_prop("sku", AAZObjectType)
                properties.set_prop("workspaceCapping", AAZObjectType)

            replication = _builder.get(".properties.replication")
            if replication is not None:
                replication.set_prop("enabled", AAZBoolType, ".replication_enabled")

            sku = _builder.get(".properties.sku")
            if sku is not None:
                sku.set_prop("capacityReservationLevel", AAZIntType, ".capacity_reservation_level")
                sku.set_prop("name", AAZStrType, ".sku_name", typ_kwargs={"flags": {"required": True}})

            workspace_capping = _builder.get(".properties.workspaceCapping")
            if workspace_capping is not None:
                workspace_capping.set_prop("dailyQuotaGb", AAZFloatType, ".quota")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    _schema_workspace_read = None

    @classmethod
    def _build_schema_workspace_read(cls, _schema):
        if cls._schema_workspace_read is not None:
            _schema.etag = cls._schema_workspace_read.etag
            _schema.id = cls._schema_workspace_read.id
            _schema.identity = cls._schema_workspace_read.identity
            _schema.location = cls._schema_workspace_read.location
            _schema.name = cls._schema_workspace_read.name
            _schema.properties = cls._schema_workspace_read.properties
            _schema.system_data = cls._schema_workspace_read.system_data
            _schema.tags = cls._schema_workspace_read.tags
            _schema.type = cls._schema_workspace_read.type
            return

        cls._schema_workspace_read = _schema_workspace_read = AAZObjectType()

        workspace_read = _schema_workspace_read
        workspace_read.etag = AAZStrType()
        workspace_read.id = AAZStrType(
            flags={"read_only": True},
        )
        workspace_read.identity = AAZIdentityObjectType()
        workspace_read.location = AAZStrType(
            flags={"required": True},
        )
        workspace_read.name = AAZStrType(
            flags={"read_only": True},
        )
        workspace_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        workspace_read.system_data = AAZObjectType(
            serialized_name="systemData",
            flags={"read_only": True},
        )
        workspace_read.tags = AAZDictType()
        workspace_read.type = AAZStrType(
            flags={"read_only": True},
        )

        identity = _schema_workspace_read.identity
        identity.principal_id = AAZStrType(
            serialized_name="principalId",
            flags={"read_only": True},
        )
        identity.tenant_id = AAZStrType(
            serialized_name="tenantId",
            flags={"read_only": True},
        )
        identity.type = AAZStrType(
            flags={"required": True},
        )
        identity.user_assigned_identities = AAZDictType(
            serialized_name="userAssignedIdentities",
        )

        user_assigned_identities = _schema_workspace_read.identity.user_assigned_identities
        user_assigned_identities.Element = AAZObjectType()

        _element = _schema_workspace_read.identity.user_assigned_identities.Element
        _element.client_id = AAZStrType(
            serialized_name="clientId",
            flags={"read_only": True},
        )
        _element.principal_id = AAZStrType(
            serialized_name="principalId",
            flags={"read_only": True},
        )

        properties = _schema_workspace_read.properties
        properties.created_date = AAZStrType(
            serialized_name="createdDate",
            flags={"read_only": True},
        )
        properties.customer_id = AAZStrType(
            serialized_name="customerId",
            flags={"read_only": True},
        )
        properties.default_data_collection_rule_resource_id = AAZStrType(
            serialized_name="defaultDataCollectionRuleResourceId",
        )
        properties.failover = AAZObjectType()
        properties.features = AAZFreeFormDictType()
        properties.force_cmk_for_query = AAZBoolType(
            serialized_name="forceCmkForQuery",
        )
        properties.modified_date = AAZStrType(
            serialized_name="modifiedDate",
            flags={"read_only": True},
        )
        properties.private_link_scoped_resources = AAZListType(
            serialized_name="privateLinkScopedResources",
            flags={"read_only": True},
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.public_network_access_for_ingestion = AAZStrType(
            serialized_name="publicNetworkAccessForIngestion",
        )
        properties.public_network_access_for_query = AAZStrType(
            serialized_name="publicNetworkAccessForQuery",
        )
        properties.replication = AAZObjectType()
        properties.retention_in_days = AAZIntType(
            serialized_name="retentionInDays",
            nullable=True,
        )
        properties.sku = AAZObjectType()
        properties.workspace_capping = AAZObjectType(
            serialized_name="workspaceCapping",
        )

        failover = _schema_workspace_read.properties.failover
        failover.last_modified_date = AAZStrType(
            serialized_name="lastModifiedDate",
            flags={"read_only": True},
        )
        failover.state = AAZStrType(
            flags={"read_only": True},
        )

        private_link_scoped_resources = _schema_workspace_read.properties.private_link_scoped_resources
        private_link_scoped_resources.Element = AAZObjectType()

        _element = _schema_workspace_read.properties.private_link_scoped_resources.Element
        _element.resource_id = AAZStrType(
            serialized_name="resourceId",
        )
        _element.scope_id = AAZStrType(
            serialized_name="scopeId",
        )

        replication = _schema_workspace_read.properties.replication
        replication.created_date = AAZStrType(
            serialized_name="createdDate",
            flags={"read_only": True},
        )
        replication.enabled = AAZBoolType()
        replication.last_modified_date = AAZStrType(
            serialized_name="lastModifiedDate",
            flags={"read_only": True},
        )
        replication.location = AAZStrType()
        replication.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )

        sku = _schema_workspace_read.properties.sku
        sku.capacity_reservation_level = AAZIntType(
            serialized_name="capacityReservationLevel",
        )
        sku.last_sku_update = AAZStrType(
            serialized_name="lastSkuUpdate",
            flags={"read_only": True},
        )
        sku.name = AAZStrType(
            flags={"required": True},
        )

        workspace_capping = _schema_workspace_read.properties.workspace_capping
        workspace_capping.daily_quota_gb = AAZFloatType(
            serialized_name="dailyQuotaGb",
        )
        workspace_capping.data_ingestion_status = AAZStrType(
            serialized_name="dataIngestionStatus",
            flags={"read_only": True},
        )
        workspace_capping.quota_next_reset_time = AAZStrType(
            serialized_name="quotaNextResetTime",
            flags={"read_only": True},
        )

        system_data = _schema_workspace_read.system_data
        system_data.created_at = AAZStrType(
            serialized_name="createdAt",
        )
        system_data.created_by = AAZStrType(
            serialized_name="createdBy",
        )
        system_data.created_by_type = AAZStrType(
            serialized_name="createdByType",
        )
        system_data.last_modified_at = AAZStrType(
            serialized_name="lastModifiedAt",
        )
        system_data.last_modified_by = AAZStrType(
            serialized_name="lastModifiedBy",
        )
        system_data.last_modified_by_type = AAZStrType(
            serialized_name="lastModifiedByType",
        )

        tags = _schema_workspace_read.tags
        tags.Element = AAZStrType()

        _schema.etag = cls._schema_workspace_read.etag
        _schema.id = cls._schema_workspace_read.id
        _schema.identity = cls._schema_workspace_read.identity
        _schema.location = cls._schema_workspace_read.location
        _schema.name = cls._schema_workspace_read.name
        _schema.properties = cls._schema_workspace_read.properties
        _schema.system_data = cls._schema_workspace_read.system_data
        _schema.tags = cls._schema_workspace_read.tags
        _schema.type = cls._schema_workspace_read.type


__all__ = ["Update"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

"""The output transformers for monitor commands. The global import should be limited to improve performance."""


def _item_to_ordered_dict(item, *properties):
    from collections import OrderedDict
    result = OrderedDict()

    for p in properties:
        if isinstance(p, tuple):
            property_name, display_name = p
        else:
            property_name = display_name = str(p)

        value = item.get(property_name)

        if isinstance(value, str):
            result[display_name] = value
        elif isinstance(value, list):
            result[display_name] = str(len(value))
        elif value is not None:
            result[display_name] = str(value)

    return result


def _generic_table_convert(source, row_convert_fn):
    if not isinstance(source, list):
        source = [source]

    return [row_convert_fn(each) for each in source]


def action_group_list_table(results):
    if not isinstance(results, list):
        results = [results]

    output_results = []
    for result in results:
        data = _item_to_ordered_dict(result, 'name', 'resourceGroup', 'groupShortName', 'enabled', 'location',
                                     ('emailReceivers', 'email'), ('smsReceivers', 'sms'),
                                     ('webhookReceivers', 'webhook'), ('armRoleReceivers', 'armrole'),
                                     ('azureAppPushReceivers', 'azureapppush'), ('itsmReceivers', 'itsm'),
                                     ('automationRunbookReceivers', 'automationrunbook'), ('voiceReceivers', 'voice'),
                                     ('logicAppReceivers', 'logicapp'), ('azureFunctionReceivers', 'azurefunction'),
                                     ('eventHubReceivers', 'eventhub'))

        output_results.append(data)

    return output_results


def metrics_definitions_table(results):
    def row_convert(item):
        from collections import OrderedDict
        result = OrderedDict()
        result['Display Name'] = item['name']['localizedValue']
        result['Metric Name'] = item['name']['value']
        result['Unit'] = item['unit']
        result['Type'] = item['primaryAggregationType']
        result['Dimension Required'] = 'True' if item['isDimensionRequired'] else 'False'
        result['Dimensions'] = ', '.join(d['value'] for d in item.get('dimensions', []) or [])

        return result

    return _generic_table_convert(results, row_convert)


def metrics_namespaces_table(results):
    def row_convert(item):
        from collections import OrderedDict
        result = OrderedDict()
        result['Classification'] = item['classification']
        result['Metric Namespace Name'] = item['properties']['metricNamespaceName']
        return result
    return _generic_table_convert(results, row_convert)


def metrics_table(results):
    from collections import OrderedDict

    def from_time(time_string):
        from datetime import datetime
        try:
            return datetime.strptime(time_string, '%Y-%m-%dT%H:%M:%S+00:00').strftime('%Y-%m-%d %H:%M:%S')
        except ValueError:
            return time_string

    retval = []
    for value_group in results['value']:
        name = value_group['name']['localizedValue']
        for series in value_group['timeseries']:
            metadata = {m['name']['localizedValue']: m['value'] for m in series['metadatavalues']}

            for data in series['data']:
                row = OrderedDict()
                row['Timestamp'] = from_time(data['timeStamp'])
                row['Name'] = name
                for metadata_name, metadata_value in metadata.items():
                    row[metadata_name] = metadata_value

                for field in data:
                    if field == 'timeStamp':
                        continue
                    row[field] = data[field]
                retval.append(row)
    return retval



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: disable=too-many-lines
# pylint: disable=too-many-statements

from knack.log import get_logger


logger = get_logger(__name__)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.log import get_logger

logger = get_logger(__name__)


def _get_current_user_object_id(graph_client):
    # Calling this API with a service principal raises GraphError:
    #   /me request is only valid with delegated authentication flow.
    current_user = graph_client.signed_in_user_get()
    return current_user['id']


def _get_object_id_by_spn(graph_client, spn):
    accounts = list(graph_client.service_principal_list(filter="servicePrincipalNames/any(c:c eq '{}')".format(spn)))
    # 2+ matches should never happen, so we only check 'no match' here
    if not accounts:
        logger.warning("Unable to find service principal with spn '%s'", spn)
        return None
    return accounts[0]['id']


def _get_object_id_by_upn(graph_client, upn):
    accounts = list(graph_client.user_list(filter="userPrincipalName eq '{}'".format(upn)))
    # 2+ matches should never happen, so we only check 'no match' here
    if not accounts:
        logger.warning("Unable to find user with upn '%s'", upn)
        return None
    return accounts[0]['id']


def _get_object_id_from_subscription(graph_client, subscription):
    if subscription['user']['type'] == 'user':
        return _get_object_id_by_upn(graph_client, subscription['user']['name'])
    if subscription['user']['type'] == 'servicePrincipal':
        return _get_object_id_by_spn(graph_client, subscription['user']['name'])
    return None


def _get_object_id(graph_client, spn=None, upn=None):
    if spn:
        return _get_object_id_by_spn(graph_client, spn)
    if upn:
        return _get_object_id_by_upn(graph_client, upn)
    return None


def get_object_id(cli_ctx, spn=None, upn=None):
    from azure.cli.command_modules.role import graph_client_factory
    graph_client = graph_client_factory(cli_ctx)
    return _get_object_id(graph_client, spn=spn, upn=upn)


def get_current_identity_object_id(cli_ctx):
    """This function tries to get the current identity's object ID following below steps:

    1. First try to resolve with /me API: https://learn.microsoft.com/en-us/graph/api/user-get
    2. If failed, try to resolve with either
      - /users API: https://learn.microsoft.com/en-us/graph/api/user-list
      - /servicePrincipals API: https://learn.microsoft.com/en-us/graph/api/serviceprincipal-list

    If all of these attempts fail, return None.
    """
    from azure.cli.command_modules.role import graph_client_factory, GraphError
    graph_client = graph_client_factory(cli_ctx)

    try:
        return _get_current_user_object_id(graph_client)
    except GraphError:
        from azure.cli.core._profile import Profile
        profile = Profile(cli_ctx)
        # To align with _get_current_user_object_id, only look up the current upn/spn, so
        # cli_ctx.data['subscription_id'] should not be used in get_subscription.
        # Otherwise, it may result in looking up a upn/spn different from the current login context.
        subscription = profile.get_subscription()
        return _get_object_id_from_subscription(graph_client, subscription)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network watcher show-next-hop",
)
class ShowNextHop(AAZCommand):
    """Get information on the `next hop` of a VM.

    Requires that Network Watcher is enabled for the region in which the VM is located. For more information about show-next-hop visit https://learn.microsoft.com/en-us/azure/network-watcher/diagnose-vm-network-routing-problem-cli.

    :example: Get the next hop from a VMs assigned IP address to a destination at 10.1.0.4.
        az network watcher show-next-hop -g MyResourceGroup --vm MyVm --source-ip 10.0.0.4 --dest-ip 10.1.0.4
    """

    _aaz_info = {
        "version": "2022-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/networkwatchers/{}/nexthop", "2022-01-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.watcher_name = AAZStrArg(
            options=["--watcher-name"],
            help="Name of the network watcher.",
            required=True,
            id_part="name",
        )
        _args_schema.watcher_rg = AAZResourceGroupNameArg(
            options=["--watcher-rg"],
            help="Name of the resource group the watcher is in.",
            required=True,
        )
        _args_schema.dest_ip = AAZStrArg(
            options=["--dest-ip"],
            help="Destination IPv4 address.",
            required=True,
        )
        _args_schema.source_ip = AAZStrArg(
            options=["--source-ip"],
            help="Source IPv4 address.",
            required=True,
        )
        _args_schema.nic = AAZStrArg(
            options=["--nic"],
            help="Name or ID of the NIC resource to test. If the VM has multiple NICs and IP forwarding is enabled on any of them, this parameter is required.",
        )
        _args_schema.vm = AAZStrArg(
            options=["--vm"],
            help="Name or ID of the VM to target. If the name of the VM is provided, the `--resource-group` is required.",
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.NetworkWatchersGetNextHop(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class NetworkWatchersGetNextHop(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkWatchers/{networkWatcherName}/nextHop",
                **self.url_parameters
            )

        @property
        def method(self):
            return "POST"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "networkWatcherName", self.ctx.args.watcher_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.watcher_rg,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("destinationIPAddress", AAZStrType, ".dest_ip", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("sourceIPAddress", AAZStrType, ".source_ip", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("targetNicResourceId", AAZStrType, ".nic")
            _builder.set_prop("targetResourceId", AAZStrType, ".vm", typ_kwargs={"flags": {"required": True}})

            return self.serialize_content(_content_value)

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _ShowNextHopHelper._build_schema_next_hop_result_read(cls._schema_on_200)

            return cls._schema_on_200


class _ShowNextHopHelper:
    """Helper class for ShowNextHop"""

    _schema_next_hop_result_read = None

    @classmethod
    def _build_schema_next_hop_result_read(cls, _schema):
        if cls._schema_next_hop_result_read is not None:
            _schema.next_hop_ip_address = cls._schema_next_hop_result_read.next_hop_ip_address
            _schema.next_hop_type = cls._schema_next_hop_result_read.next_hop_type
            _schema.route_table_id = cls._schema_next_hop_result_read.route_table_id
            return

        cls._schema_next_hop_result_read = _schema_next_hop_result_read = AAZObjectType()

        next_hop_result_read = _schema_next_hop_result_read
        next_hop_result_read.next_hop_ip_address = AAZStrType(
            serialized_name="nextHopIpAddress",
        )
        next_hop_result_read.next_hop_type = AAZStrType(
            serialized_name="nextHopType",
        )
        next_hop_result_read.route_table_id = AAZStrType(
            serialized_name="routeTableId",
        )

        _schema.next_hop_ip_address = cls._schema_next_hop_result_read.next_hop_ip_address
        _schema.next_hop_type = cls._schema_next_hop_result_read.next_hop_type
        _schema.route_table_id = cls._schema_next_hop_result_read.route_table_id


__all__ = ["ShowNextHop"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class Create(AAZCommand):
    """Create a route server.

    :example: Create a route server.
        az network routeserver create --resource-group myresourcegroup --name myrouteserver --hosted-subnet my_subnet_id --public-ip-address my_public_ip
    """

    _aaz_info = {
        "version": "2022-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/virtualhubs/{}", "2022-01-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="Name of the route server.",
            required=True,
        )
        _args_schema.location = AAZResourceLocationArg(
            help="Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.",
            required=True,
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.hub_routing_preference = AAZStrArg(
            options=["--hub-routing-preference"],
            help="Routing preference of the route server.",
            enum={"ASPath": "ASPath", "ExpressRoute": "ExpressRoute", "VpnGateway": "VpnGateway"},
        )
        _args_schema.sku = AAZStrArg(
            options=["--sku"],
            help="SKU of the route server.",
        )
        _args_schema.auto_scale_config = AAZObjectArg(
            options=["--auto-scale-config"],
            help="The VirtualHub Router autoscale configuration.",
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            help="Space-separated tags: key[=value] [key[=value] ...].",
        )

        auto_scale_config = cls._args_schema.auto_scale_config
        auto_scale_config.min_capacity = AAZIntArg(
            options=["min-capacity"],
            help="The minimum number of scale units for VirtualHub Router.",
            fmt=AAZIntArgFormat(
                minimum=0,
            ),
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg()

        # define Arg Group "Properties"

        # define Arg Group "VirtualHubParameters"
        return cls._args_schema

    _args_sub_resource_create = None

    @classmethod
    def _build_args_sub_resource_create(cls, _schema):
        if cls._args_sub_resource_create is not None:
            _schema.id = cls._args_sub_resource_create.id
            return

        cls._args_sub_resource_create = AAZObjectArg()

        sub_resource_create = cls._args_sub_resource_create
        sub_resource_create.id = AAZStrArg(
            options=["id"],
            help="Resource ID.",
        )

        _schema.id = cls._args_sub_resource_create.id

    def _execute_operations(self):
        self.pre_operations()
        yield self.VirtualHubsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class VirtualHubsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/virtualHubs/{virtualHubName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "virtualHubName", self.ctx.args.name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("location", AAZStrType, ".location", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("hubRoutingPreference", AAZStrType, ".hub_routing_preference")
                properties.set_prop("sku", AAZStrType, ".sku")
                properties.set_prop("virtualRouterAutoScaleConfiguration", AAZObjectType, ".auto_scale_config")

            virtual_router_auto_scale_configuration = _builder.get(".properties.virtualRouterAutoScaleConfiguration")
            if virtual_router_auto_scale_configuration is not None:
                virtual_router_auto_scale_configuration.set_prop("minCapacity", AAZIntType, ".min_capacity")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.id = AAZStrType()
            _schema_on_200_201.kind = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200_201.tags = AAZDictType()
            _schema_on_200_201.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200_201.properties
            properties.address_prefix = AAZStrType(
                serialized_name="addressPrefix",
            )
            properties.allow_branch_to_branch_traffic = AAZBoolType(
                serialized_name="allowBranchToBranchTraffic",
            )
            properties.azure_firewall = AAZObjectType(
                serialized_name="azureFirewall",
            )
            _CreateHelper._build_schema_sub_resource_read(properties.azure_firewall)
            properties.bgp_connections = AAZListType(
                serialized_name="bgpConnections",
                flags={"read_only": True},
            )
            properties.express_route_gateway = AAZObjectType(
                serialized_name="expressRouteGateway",
            )
            _CreateHelper._build_schema_sub_resource_read(properties.express_route_gateway)
            properties.hub_routing_preference = AAZStrType(
                serialized_name="hubRoutingPreference",
            )
            properties.ip_configurations = AAZListType(
                serialized_name="ipConfigurations",
                flags={"read_only": True},
            )
            properties.p2_s_vpn_gateway = AAZObjectType(
                serialized_name="p2SVpnGateway",
            )
            _CreateHelper._build_schema_sub_resource_read(properties.p2_s_vpn_gateway)
            properties.preferred_routing_gateway = AAZStrType(
                serialized_name="preferredRoutingGateway",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.route_table = AAZObjectType(
                serialized_name="routeTable",
            )
            properties.routing_state = AAZStrType(
                serialized_name="routingState",
                flags={"read_only": True},
            )
            properties.security_partner_provider = AAZObjectType(
                serialized_name="securityPartnerProvider",
            )
            _CreateHelper._build_schema_sub_resource_read(properties.security_partner_provider)
            properties.security_provider_name = AAZStrType(
                serialized_name="securityProviderName",
            )
            properties.sku = AAZStrType()
            properties.virtual_hub_route_table_v2s = AAZListType(
                serialized_name="virtualHubRouteTableV2s",
            )
            properties.virtual_router_asn = AAZIntType(
                serialized_name="virtualRouterAsn",
            )
            properties.virtual_router_auto_scale_configuration = AAZObjectType(
                serialized_name="virtualRouterAutoScaleConfiguration",
            )
            properties.virtual_router_ips = AAZListType(
                serialized_name="virtualRouterIps",
            )
            properties.virtual_wan = AAZObjectType(
                serialized_name="virtualWan",
            )
            _CreateHelper._build_schema_sub_resource_read(properties.virtual_wan)
            properties.vpn_gateway = AAZObjectType(
                serialized_name="vpnGateway",
            )
            _CreateHelper._build_schema_sub_resource_read(properties.vpn_gateway)

            bgp_connections = cls._schema_on_200_201.properties.bgp_connections
            bgp_connections.Element = AAZObjectType()
            _CreateHelper._build_schema_sub_resource_read(bgp_connections.Element)

            ip_configurations = cls._schema_on_200_201.properties.ip_configurations
            ip_configurations.Element = AAZObjectType()
            _CreateHelper._build_schema_sub_resource_read(ip_configurations.Element)

            route_table = cls._schema_on_200_201.properties.route_table
            route_table.routes = AAZListType()

            routes = cls._schema_on_200_201.properties.route_table.routes
            routes.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.route_table.routes.Element
            _element.address_prefixes = AAZListType(
                serialized_name="addressPrefixes",
            )
            _element.next_hop_ip_address = AAZStrType(
                serialized_name="nextHopIpAddress",
            )

            address_prefixes = cls._schema_on_200_201.properties.route_table.routes.Element.address_prefixes
            address_prefixes.Element = AAZStrType()

            virtual_hub_route_table_v2s = cls._schema_on_200_201.properties.virtual_hub_route_table_v2s
            virtual_hub_route_table_v2s.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.virtual_hub_route_table_v2s.Element
            _element.etag = AAZStrType(
                flags={"read_only": True},
            )
            _element.id = AAZStrType()
            _element.name = AAZStrType()
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )

            properties = cls._schema_on_200_201.properties.virtual_hub_route_table_v2s.Element.properties
            properties.attached_connections = AAZListType(
                serialized_name="attachedConnections",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.routes = AAZListType()

            attached_connections = cls._schema_on_200_201.properties.virtual_hub_route_table_v2s.Element.properties.attached_connections
            attached_connections.Element = AAZStrType()

            routes = cls._schema_on_200_201.properties.virtual_hub_route_table_v2s.Element.properties.routes
            routes.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.virtual_hub_route_table_v2s.Element.properties.routes.Element
            _element.destination_type = AAZStrType(
                serialized_name="destinationType",
            )
            _element.destinations = AAZListType()
            _element.next_hop_type = AAZStrType(
                serialized_name="nextHopType",
            )
            _element.next_hops = AAZListType(
                serialized_name="nextHops",
            )

            destinations = cls._schema_on_200_201.properties.virtual_hub_route_table_v2s.Element.properties.routes.Element.destinations
            destinations.Element = AAZStrType()

            next_hops = cls._schema_on_200_201.properties.virtual_hub_route_table_v2s.Element.properties.routes.Element.next_hops
            next_hops.Element = AAZStrType()

            virtual_router_auto_scale_configuration = cls._schema_on_200_201.properties.virtual_router_auto_scale_configuration
            virtual_router_auto_scale_configuration.min_capacity = AAZIntType(
                serialized_name="minCapacity",
            )

            virtual_router_ips = cls._schema_on_200_201.properties.virtual_router_ips
            virtual_router_ips.Element = AAZStrType()

            tags = cls._schema_on_200_201.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""

    @classmethod
    def _build_schema_sub_resource_create(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("id", AAZStrType, ".id")

    _schema_sub_resource_read = None

    @classmethod
    def _build_schema_sub_resource_read(cls, _schema):
        if cls._schema_sub_resource_read is not None:
            _schema.id = cls._schema_sub_resource_read.id
            return

        cls._schema_sub_resource_read = _schema_sub_resource_read = AAZObjectType()

        sub_resource_read = _schema_sub_resource_read
        sub_resource_read.id = AAZStrType()

        _schema.id = cls._schema_sub_resource_read.id


__all__ = ["Create"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network virtual-appliance connection update",
)
class Update(AAZCommand):
    """Update existing connection to Network Virtual Appliance

    :example: Update NVA connection routing configuration - change associated route table
        az network virtual-appliance connection update --name defaultConnection  --nva MyNva -g MyRG --subscription <subId> --associated-route-table "{'id': '/subscriptions/<subId>/resourceGroups/<MyRG>/providers/Microsoft.Network/virtualHubs/<vhubName>/hubRouteTables/<customRouteTable>'}"

    :example: Update NVA connection routing configuration - add Propagated route table Labels
        az network virtual-appliance connection update --name defaultConnection  --nva MyNva -g MyRG --subscription <subId>  --labels [label1,label2]

    :example: Update NVA connection routing configuration - add propagated route table id
        az network virtual-appliance connection update --name defaultConnection  --nva MyNva -g MyRG --subscription <subId>   --propagated "[{'id':'<routeTable1id>'},{'id':'<routeTable2id>'}]"
    """

    _aaz_info = {
        "version": "2023-06-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/networkvirtualappliances/{}/networkvirtualapplianceconnections/{}", "2023-06-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="The name of the NVA connection.",
            required=True,
            id_part="child_name_1",
        )
        _args_schema.virtual_appliance_name = AAZStrArg(
            options=["--nva", "--virtual-appliance-name"],
            help="The name of the Network Virtual Appliance.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "NetworkVirtualApplianceConnectionParameters"

        # define Arg Group "PropagatedRouteTables"

        _args_schema = cls._args_schema
        _args_schema.propagated_route_table_ids = AAZListArg(
            options=["--propagated", "--propagated-route-table-ids"],
            arg_group="PropagatedRouteTables",
            help="List of resource id of propagated route tables.",
            nullable=True,
        )
        _args_schema.propagated_route_table_labels = AAZListArg(
            options=["--labels", "--propagated-route-table-labels"],
            arg_group="PropagatedRouteTables",
            help="The list of labels.",
            nullable=True,
        )

        propagated_route_table_ids = cls._args_schema.propagated_route_table_ids
        propagated_route_table_ids.Element = AAZObjectArg(
            nullable=True,
        )
        cls._build_args_sub_resource_update(propagated_route_table_ids.Element)

        propagated_route_table_labels = cls._args_schema.propagated_route_table_labels
        propagated_route_table_labels.Element = AAZStrArg(
            nullable=True,
        )

        # define Arg Group "Properties"

        # define Arg Group "RoutingConfiguration"

        _args_schema = cls._args_schema
        _args_schema.associated_route_table = AAZObjectArg(
            options=["--associated-route-table"],
            arg_group="RoutingConfiguration",
            help="The resource id RouteTable associated with this RoutingConfiguration.",
            nullable=True,
        )
        cls._build_args_sub_resource_update(_args_schema.associated_route_table)
        _args_schema.inbound_route_map = AAZObjectArg(
            options=["--inbound-route-map"],
            arg_group="RoutingConfiguration",
            help="The resource id of the RouteMap associated with this RoutingConfiguration for inbound learned routes.",
            nullable=True,
        )
        cls._build_args_sub_resource_update(_args_schema.inbound_route_map)
        _args_schema.outbound_route_map = AAZObjectArg(
            options=["--outbound-route-map"],
            arg_group="RoutingConfiguration",
            help="The resource id of theRouteMap associated with this RoutingConfiguration for outbound advertised routes.",
            nullable=True,
        )
        cls._build_args_sub_resource_update(_args_schema.outbound_route_map)
        return cls._args_schema

    _args_sub_resource_update = None

    @classmethod
    def _build_args_sub_resource_update(cls, _schema):
        if cls._args_sub_resource_update is not None:
            _schema.id = cls._args_sub_resource_update.id
            return

        cls._args_sub_resource_update = AAZObjectArg(
            nullable=True,
        )

        sub_resource_update = cls._args_sub_resource_update
        sub_resource_update.id = AAZStrArg(
            options=["id"],
            help="Resource ID.",
            nullable=True,
        )

        _schema.id = cls._args_sub_resource_update.id

    def _execute_operations(self):
        self.pre_operations()
        self.NetworkVirtualApplianceConnectionsGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        yield self.NetworkVirtualApplianceConnectionsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class NetworkVirtualApplianceConnectionsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkVirtualAppliances/{networkVirtualApplianceName}/networkVirtualApplianceConnections/{connectionName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "connectionName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "networkVirtualApplianceName", self.ctx.args.virtual_appliance_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2023-06-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_network_virtual_appliance_connection_read(cls._schema_on_200)

            return cls._schema_on_200

    class NetworkVirtualApplianceConnectionsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkVirtualAppliances/{networkVirtualApplianceName}/networkVirtualApplianceConnections/{connectionName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "connectionName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "networkVirtualApplianceName", self.ctx.args.virtual_appliance_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2023-06-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _UpdateHelper._build_schema_network_virtual_appliance_connection_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("properties", AAZObjectType)

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("name", AAZStrType, ".name")
                properties.set_prop("routingConfiguration", AAZObjectType)

            routing_configuration = _builder.get(".properties.routingConfiguration")
            if routing_configuration is not None:
                _UpdateHelper._build_schema_sub_resource_update(routing_configuration.set_prop("associatedRouteTable", AAZObjectType, ".associated_route_table"))
                _UpdateHelper._build_schema_sub_resource_update(routing_configuration.set_prop("inboundRouteMap", AAZObjectType, ".inbound_route_map"))
                _UpdateHelper._build_schema_sub_resource_update(routing_configuration.set_prop("outboundRouteMap", AAZObjectType, ".outbound_route_map"))
                routing_configuration.set_prop("propagatedRouteTables", AAZObjectType)

            propagated_route_tables = _builder.get(".properties.routingConfiguration.propagatedRouteTables")
            if propagated_route_tables is not None:
                propagated_route_tables.set_prop("ids", AAZListType, ".propagated_route_table_ids")
                propagated_route_tables.set_prop("labels", AAZListType, ".propagated_route_table_labels")

            ids = _builder.get(".properties.routingConfiguration.propagatedRouteTables.ids")
            if ids is not None:
                _UpdateHelper._build_schema_sub_resource_update(ids.set_elements(AAZObjectType, "."))

            labels = _builder.get(".properties.routingConfiguration.propagatedRouteTables.labels")
            if labels is not None:
                labels.set_elements(AAZStrType, ".")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    @classmethod
    def _build_schema_sub_resource_update(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("id", AAZStrType, ".id")

    _schema_network_virtual_appliance_connection_read = None

    @classmethod
    def _build_schema_network_virtual_appliance_connection_read(cls, _schema):
        if cls._schema_network_virtual_appliance_connection_read is not None:
            _schema.id = cls._schema_network_virtual_appliance_connection_read.id
            _schema.properties = cls._schema_network_virtual_appliance_connection_read.properties
            return

        cls._schema_network_virtual_appliance_connection_read = _schema_network_virtual_appliance_connection_read = AAZObjectType()

        network_virtual_appliance_connection_read = _schema_network_virtual_appliance_connection_read
        network_virtual_appliance_connection_read.id = AAZStrType()
        network_virtual_appliance_connection_read.properties = AAZObjectType()

        properties = _schema_network_virtual_appliance_connection_read.properties
        properties.asn = AAZIntType()
        properties.bgp_peer_address = AAZListType(
            serialized_name="bgpPeerAddress",
        )
        properties.enable_internet_security = AAZBoolType(
            serialized_name="enableInternetSecurity",
        )
        properties.name = AAZStrType()
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.routing_configuration = AAZObjectType(
            serialized_name="routingConfiguration",
        )
        properties.tunnel_identifier = AAZIntType(
            serialized_name="tunnelIdentifier",
        )

        bgp_peer_address = _schema_network_virtual_appliance_connection_read.properties.bgp_peer_address
        bgp_peer_address.Element = AAZStrType()

        routing_configuration = _schema_network_virtual_appliance_connection_read.properties.routing_configuration
        routing_configuration.associated_route_table = AAZObjectType(
            serialized_name="associatedRouteTable",
        )
        cls._build_schema_sub_resource_read(routing_configuration.associated_route_table)
        routing_configuration.inbound_route_map = AAZObjectType(
            serialized_name="inboundRouteMap",
        )
        cls._build_schema_sub_resource_read(routing_configuration.inbound_route_map)
        routing_configuration.outbound_route_map = AAZObjectType(
            serialized_name="outboundRouteMap",
        )
        cls._build_schema_sub_resource_read(routing_configuration.outbound_route_map)
        routing_configuration.propagated_route_tables = AAZObjectType(
            serialized_name="propagatedRouteTables",
        )
        routing_configuration.vnet_routes = AAZObjectType(
            serialized_name="vnetRoutes",
        )

        propagated_route_tables = _schema_network_virtual_appliance_connection_read.properties.routing_configuration.propagated_route_tables
        propagated_route_tables.ids = AAZListType()
        propagated_route_tables.labels = AAZListType()

        ids = _schema_network_virtual_appliance_connection_read.properties.routing_configuration.propagated_route_tables.ids
        ids.Element = AAZObjectType()
        cls._build_schema_sub_resource_read(ids.Element)

        labels = _schema_network_virtual_appliance_connection_read.properties.routing_configuration.propagated_route_tables.labels
        labels.Element = AAZStrType()

        vnet_routes = _schema_network_virtual_appliance_connection_read.properties.routing_configuration.vnet_routes
        vnet_routes.bgp_connections = AAZListType(
            serialized_name="bgpConnections",
            flags={"read_only": True},
        )
        vnet_routes.static_routes = AAZListType(
            serialized_name="staticRoutes",
        )
        vnet_routes.static_routes_config = AAZObjectType(
            serialized_name="staticRoutesConfig",
        )

        bgp_connections = _schema_network_virtual_appliance_connection_read.properties.routing_configuration.vnet_routes.bgp_connections
        bgp_connections.Element = AAZObjectType()
        cls._build_schema_sub_resource_read(bgp_connections.Element)

        static_routes = _schema_network_virtual_appliance_connection_read.properties.routing_configuration.vnet_routes.static_routes
        static_routes.Element = AAZObjectType()

        _element = _schema_network_virtual_appliance_connection_read.properties.routing_configuration.vnet_routes.static_routes.Element
        _element.address_prefixes = AAZListType(
            serialized_name="addressPrefixes",
        )
        _element.name = AAZStrType()
        _element.next_hop_ip_address = AAZStrType(
            serialized_name="nextHopIpAddress",
        )

        address_prefixes = _schema_network_virtual_appliance_connection_read.properties.routing_configuration.vnet_routes.static_routes.Element.address_prefixes
        address_prefixes.Element = AAZStrType()

        static_routes_config = _schema_network_virtual_appliance_connection_read.properties.routing_configuration.vnet_routes.static_routes_config
        static_routes_config.propagate_static_routes = AAZBoolType(
            serialized_name="propagateStaticRoutes",
            flags={"read_only": True},
        )
        static_routes_config.vnet_local_route_override_criteria = AAZStrType(
            serialized_name="vnetLocalRouteOverrideCriteria",
        )

        _schema.id = cls._schema_network_virtual_appliance_connection_read.id
        _schema.properties = cls._schema_network_virtual_appliance_connection_read.properties

    _schema_sub_resource_read = None

    @classmethod
    def _build_schema_sub_resource_read(cls, _schema):
        if cls._schema_sub_resource_read is not None:
            _schema.id = cls._schema_sub_resource_read.id
            return

        cls._schema_sub_resource_read = _schema_sub_resource_read = AAZObjectType()

        sub_resource_read = _schema_sub_resource_read
        sub_resource_read.id = AAZStrType()

        _schema.id = cls._schema_sub_resource_read.id


__all__ = ["Update"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core import AzCommandsLoader

import azure.cli.command_modules.batch._help  # pylint: disable=unused-import
from azure.cli.command_modules.batch._exception_handler import batch_exception_handler
from azure.cli.command_modules.batch._command_type import BatchCommandGroup


class BatchCommandsLoader(AzCommandsLoader):

    def __init__(self, cli_ctx=None):
        from azure.cli.core.commands import CliCommandType
        from azure.cli.core.profiles import ResourceType
        batch_custom = CliCommandType(
            operations_tmpl='azure.cli.command_modules.batch.custom#{}',
            exception_handler=batch_exception_handler)
        super().__init__(cli_ctx=cli_ctx,
                         custom_command_type=batch_custom,
                         command_group_cls=BatchCommandGroup,
                         resource_type=ResourceType.MGMT_BATCH)
        self.module_name = __name__

    def load_command_table(self, args):
        from azure.cli.command_modules.batch.commands import load_command_table
        load_command_table(self, args)
        return self.command_table

    def load_arguments(self, command):
        from azure.cli.command_modules.batch._params import load_arguments
        load_arguments(self, command)


COMMAND_LOADER_CLS = BatchCommandsLoader



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long,too-many-nested-blocks,too-many-lines,too-many-return-statements

import io
import json
from itertools import chain
from json import JSONDecodeError
from urllib.parse import urlparse

import chardet
import javaproperties
import yaml
from knack.log import get_logger
from knack.util import CLIError

from azure.core.exceptions import HttpResponseError
from azure.keyvault.secrets._shared import parse_key_vault_id
from azure.appconfiguration import ConfigurationSetting, ResourceReadOnlyError
from azure.cli.core.util import user_confirmation
from azure.cli.core.azclierror import (
    AzureInternalError,
    AzureResponseError,
    FileOperationError,
    InvalidArgumentValueError,
    RequiredArgumentMissingError,
    ValidationError,
)
from azure.cli.command_modules.appconfig._kv_helpers import (
    __flatten_json_key_value,
    __is_key_vault_ref,
    __read_kv_from_config_store,
    __write_configuration_setting_to_config_store,
)

from ._constants import (
    FeatureFlagConstants,
    KeyVaultConstants,
    SearchFilterOptions,
    KVSetConstants,
    ImportExportProfiles,
    AppServiceConstants,
    JsonDiff,
    CompareFieldsMap,
    ImportMode,
)
from ._diff_utils import KVComparer, print_preview
from ._utils import (
    is_json_content_type,
    validate_feature_flag_name,
    validate_feature_flag_key,
)
from ._models import KeyValue
from ._featuremodels import (
    FeatureAllocation,
    FeatureFilter,
    FeatureVariant,
    FeatureTelemetry,
    is_feature_flag,
    FeatureFlagValue,
    FeatureManagementReservedKeywords,
    convert_string_to_bool
)

logger = get_logger(__name__)


FEATURE_FLAG_PROPERTIES = {
    FeatureFlagConstants.ID,
    FeatureFlagConstants.DESCRIPTION,
    FeatureFlagConstants.ENABLED,
    FeatureFlagConstants.CONDITIONS,
    FeatureFlagConstants.ALLOCATION,
    FeatureFlagConstants.VARIANTS,
    FeatureFlagConstants.DISPLAY_NAME,
    FeatureFlagConstants.TELEMETRY,
}


def __read_with_appropriate_encoding(file_path, format_):
    config_data = {}
    default_encoding = "utf-8"
    detected_encoding = __check_file_encoding(file_path)

    try:
        with io.open(file_path, "r", encoding=default_encoding) as config_file:
            if format_ == "json":
                config_data = json.load(config_file)
                # Only accept json objects
                if not isinstance(config_data, (dict, list)):
                    raise ValueError(
                        "Json object required but type '{}' was given.".format(
                            type(config_data).__name__
                        )
                    )

            elif format_ == "yaml":
                for yaml_data in list(yaml.safe_load_all(config_file)):
                    config_data.update(yaml_data)

            elif format_ == "properties":
                config_data = javaproperties.load(config_file)
                logger.debug(
                    "Importing feature flags from a properties file is not supported. If properties file contains feature flags, they will be imported as regular key-values."
                )

    except (UnicodeDecodeError, json.JSONDecodeError):
        if detected_encoding == default_encoding:
            raise

        with io.open(file_path, "r", encoding=detected_encoding) as config_file:
            if format_ == "json":
                config_data = json.load(config_file)

            elif format_ == "yaml":
                for yaml_data in list(yaml.safe_load_all(config_file)):
                    config_data.update(yaml_data)

            elif format_ == "properties":
                config_data = javaproperties.load(config_file)
                logger.debug(
                    "Importing feature flags from a properties file is not supported. If properties file contains feature flags, they will be imported as regular key-values."
                )

    return config_data


def __read_features_from_file(file_path, format_):
    features_dict = {}

    if format_ == "properties":
        logger.warning(
            "Importing feature flags from a properties file is not supported. If properties file contains feature flags, they will be imported as regular key-values."
        )
        return features_dict

    try:
        config_data = __read_with_appropriate_encoding(file_path, format_)
        found_dotnet_schema = False
        found_ms_fm_schema = False
        dotnet_schema_keyword = None
        for keywordset in (
            FeatureManagementReservedKeywords.PASCAL,
            FeatureManagementReservedKeywords.CAMEL,
            FeatureManagementReservedKeywords.HYPHEN,
        ):
            # find the occurrences of feature management section in file.
            if keywordset.feature_management in config_data:
                if found_dotnet_schema:
                    raise FileOperationError(
                        'Unable to proceed because file contains multiple sections corresponding to "Feature Management".'
                    )
                features_dict[keywordset.feature_management] = config_data[
                    keywordset.feature_management
                ]
                dotnet_schema_keyword = keywordset.feature_management
                del config_data[keywordset.feature_management]
                found_dotnet_schema = True

        ms_feature_management_keyword = FeatureManagementReservedKeywords.UNDERSCORE.feature_management

        if ms_feature_management_keyword in config_data:
            found_dotnet_schema_within_ms_fm_schema = any(key != FeatureFlagConstants.FEATURE_FLAGS_KEY for key in config_data[ms_feature_management_keyword].keys())

            if found_dotnet_schema and found_dotnet_schema_within_ms_fm_schema:
                raise FileOperationError(
                    "Data contains an already defined section with the key %s."
                    % (dotnet_schema_keyword)
                )

            if (
                FeatureFlagConstants.FEATURE_FLAGS_KEY
                in config_data[ms_feature_management_keyword]
            ):
                found_ms_fm_schema = True

            if found_dotnet_schema_within_ms_fm_schema:
                found_dotnet_schema = True
                dotnet_schema_keyword = ms_feature_management_keyword

            features_dict[ms_feature_management_keyword] = config_data[
                ms_feature_management_keyword
            ]
            del config_data[ms_feature_management_keyword]

    except ValueError as ex:
        raise FileOperationError(
            "The feature management section of input is not a well formatted %s file.\nException: %s"
            % (format_, ex)
        )
    except yaml.YAMLError as ex:
        raise FileOperationError(
            "The feature management section of input is not a well formatted YAML file.\nException: %s"
            % (ex)
        )
    except OSError:
        raise FileOperationError("File is not available.")

    # features_dict contains all features that need to be converted to KeyValue format now
    return __convert_feature_dict_to_keyvalue_list(
        dotnet_schema_keyword, found_ms_fm_schema, features_dict
    )


def __convert_feature_dict_to_keyvalue_list(
    dotnet_schema_keyword, found_ms_fm_schema, features_dict
):
    # pylint: disable=too-many-nested-blocks
    key_values = []
    feature_flags = []

    try:
        if dotnet_schema_keyword:
            for keywordset in FeatureManagementReservedKeywords.ALL:
                if keywordset.feature_management == dotnet_schema_keyword:
                    feature_management_keywords = keywordset
                    break
            dotnet_feature_management_section = __get_dotnet_feature_management_schema(
                dotnet_schema_keyword, features_dict
            )
            dotnet_feature_flags = __read_features_from_dotnet_schema(
                dotnet_feature_management_section, feature_management_keywords
            )
            feature_flags.extend(dotnet_feature_flags)

        if found_ms_fm_schema:
            feature_management_section = __get_ms_feature_management_schema(features_dict)
            ms_feature_flags = __read_features_from_msfm_schema(feature_management_section[FeatureFlagConstants.FEATURE_FLAGS_KEY])
            # Check if the featureFlag with the same id already exists
            # Replace the existing flag with the later one, the later one always wins
            for flag in ms_feature_flags:
                index_of_existing_flag = next((i for i, existing_flag in enumerate(feature_flags) if existing_flag.id == flag.id), -1)

                if index_of_existing_flag != -1:
                    feature_flags[index_of_existing_flag] = flag
                else:
                    feature_flags.append(flag)

        for feature in feature_flags:
            key = FeatureFlagConstants.FEATURE_FLAG_PREFIX + str(feature.id)
            set_kv = KeyValue(
                key=key,
                value=json.dumps(
                    feature,
                    default=lambda o: {k: v for k, v in o.__dict__.items() if v is not None},
                    ensure_ascii=False,
                ),
                content_type=FeatureFlagConstants.FEATURE_FLAG_CONTENT_TYPE,
            )
            key_values.append(set_kv)

    except Exception as exception:
        raise CLIError(
            "File contains feature flags in invalid format. " + str(exception)
        )
    return key_values


def __get_ms_feature_management_schema(features_dict):
    ms_fm_keyword = FeatureManagementReservedKeywords.UNDERSCORE.feature_management
    feature_management_section = features_dict.get(ms_fm_keyword, None)
    if feature_management_section and not isinstance(feature_management_section, dict):
        raise ValidationError("Feature management section must be a dictionary.")

    if not isinstance(feature_management_section[FeatureFlagConstants.FEATURE_FLAGS_KEY], list):
        raise ValidationError("Feature flags must be a list.")

    return feature_management_section


def __get_dotnet_feature_management_schema(
    dotnetSchemaFeatureManagementKeyWord, features_dict
):
    ms_fm_keyword = FeatureManagementReservedKeywords.UNDERSCORE.feature_management
    feature_management_section = features_dict.get(
        dotnetSchemaFeatureManagementKeyWord, None
    )

    if feature_management_section and not isinstance(feature_management_section, dict):
        raise ValidationError("Feature management section must be a dictionary.")

    if dotnetSchemaFeatureManagementKeyWord == ms_fm_keyword:
        return dict(
            (key, feature_management_section[key])
            for key in feature_management_section.keys()
            if key != FeatureFlagConstants.FEATURE_FLAGS_KEY
        )
    return feature_management_section


def __read_features_from_msfm_schema(feature_flags_list):
    feature_flags = []
    for feature in feature_flags_list:
        if validate_import_feature(feature[FeatureFlagConstants.ID]):
            feature_id = feature[FeatureFlagConstants.ID]

            new_feature = FeatureFlagValue(
                id_=str(feature_id),
                enabled=convert_string_to_bool(
                    feature.get(FeatureFlagConstants.ENABLED, False), feature_id
                ),
                description=feature.get(FeatureFlagConstants.DESCRIPTION, None),
            )
            if conditions := feature.get(FeatureFlagConstants.CONDITIONS, None):
                new_feature.conditions = conditions
                client_filters = conditions.get(FeatureFlagConstants.CLIENT_FILTERS, None)

                # Convert all filters to FeatureFilter objects
                if client_filters:
                    client_filters_list = []
                    for client_filter in client_filters:
                        # If there is a filter, it should always have a name
                        # In case it doesn't, ignore this filter
                        lowercase_filter = {k.lower(): v for k, v in client_filter.items()}
                        name = lowercase_filter.get(FeatureFlagConstants.NAME)
                        if name:
                            params = lowercase_filter.get(
                                FeatureFlagConstants.FILTER_PARAMETERS, {}
                            )
                            client_filters_list.append(FeatureFilter(name, params))
                        else:
                            logger.warning(
                                "Ignoring this filter without the %s attribute:\n%s",
                                FeatureFlagConstants.FILTER_NAME,
                                json.dumps(client_filter, indent=2, ensure_ascii=False),
                            )
                    new_feature.conditions[FeatureFlagConstants.CLIENT_FILTERS] = client_filters_list
                requirement_type = conditions.get(
                    FeatureFlagConstants.REQUIREMENT_TYPE, None
                )
                if requirement_type:
                    # Requirement type is case insensitive.
                    if requirement_type.lower() == FeatureFlagConstants.REQUIREMENT_TYPE_ALL.lower():
                        new_feature.conditions[FeatureFlagConstants.REQUIREMENT_TYPE] = FeatureFlagConstants.REQUIREMENT_TYPE_ALL
                    elif requirement_type.lower() == FeatureFlagConstants.REQUIREMENT_TYPE_ANY.lower():
                        new_feature.conditions[FeatureFlagConstants.REQUIREMENT_TYPE] = FeatureFlagConstants.REQUIREMENT_TYPE_ANY
                    else:
                        raise ValidationError(
                            f"Feature '{feature_id}' must have an Any/All requirement type."
                        )

            if allocation := feature.get(FeatureFlagConstants.ALLOCATION, None):
                new_feature.allocation = FeatureAllocation.convert_from_dict(allocation)

            if variants := feature.get(FeatureFlagConstants.VARIANTS, None):
                new_feature.variants = []
                for variant in variants:
                    if variant:
                        new_feature.variants.append(FeatureVariant.convert_from_dict(variant))

            if telemetry := feature.get(FeatureFlagConstants.TELEMETRY, None):
                new_feature.telemetry = FeatureTelemetry.convert_from_dict(telemetry, feature_id)

            feature_flags.append(new_feature)

    return feature_flags


def __read_features_from_dotnet_schema(features_dict, feature_management_keywords):
    feature_flags = []
    default_conditions = {FeatureFlagConstants.CLIENT_FILTERS: []}

    for k, v in features_dict.items():
        if validate_import_feature(feature=k):
            feature_flag_value = FeatureFlagValue(id_=str(k))

            if isinstance(v, dict):
                # This may be a conditional feature
                feature_flag_value.enabled = False
                feature_flag_value.conditions = {}
                enabled_for_found = False

                for condition, condition_value in v.items():
                    if condition == feature_management_keywords.enabled_for:
                        feature_flag_value.conditions[
                            FeatureFlagConstants.CLIENT_FILTERS
                        ] = condition_value
                        enabled_for_found = True
                    elif (
                        condition == feature_management_keywords.requirement_type and
                        condition_value
                    ):
                        if condition_value.lower() == FeatureFlagConstants.REQUIREMENT_TYPE_ALL.lower():
                            feature_flag_value.conditions[FeatureFlagConstants.REQUIREMENT_TYPE] = FeatureFlagConstants.REQUIREMENT_TYPE_ALL
                        elif condition_value.lower() == FeatureFlagConstants.REQUIREMENT_TYPE_ANY.lower():
                            feature_flag_value.conditions[FeatureFlagConstants.REQUIREMENT_TYPE] = FeatureFlagConstants.REQUIREMENT_TYPE_ANY
                        else:
                            raise ValidationError(
                                "Feature '{0}' must have an Any/All requirement type. \n".format(
                                    str(k)
                                )
                            )
                    else:
                        feature_flag_value.conditions[condition] = condition_value
                if not enabled_for_found:
                    raise ValidationError(
                        "Feature '{0}' must contain '{1}' definition or have a true/false value. \n".format(
                            str(k), feature_management_keywords.enabled_for
                        )
                    )

                if feature_flag_value.conditions[FeatureFlagConstants.CLIENT_FILTERS]:
                    feature_flag_value.enabled = True

                    for idx, val in enumerate(
                        feature_flag_value.conditions[
                            FeatureFlagConstants.CLIENT_FILTERS
                        ]
                    ):
                        # each val should be a dict with at most 2 keys (Name, Parameters) or at least 1 key (Name)
                        val = {
                            filter_key.lower(): filter_val
                            for filter_key, filter_val in val.items()
                        }
                        if not val.get(FeatureFlagConstants.NAME, None):
                            raise ValidationError(
                                "Feature filter for feature '%s' must contain a 'Name' attribute."
                                % (str(k))
                            )

                        if val[FeatureFlagConstants.NAME].lower() == "alwayson":
                            # We support alternate format for specifying always ON features
                            # "FeatureT": {"EnabledFor": [{ "Name": "AlwaysOn"}]}
                            feature_flag_value.conditions = default_conditions
                            break

                        filter_param = val.get(
                            FeatureFlagConstants.FILTER_PARAMETERS, {}
                        )
                        new_val = {
                            FeatureFlagConstants.NAME: val[FeatureFlagConstants.NAME]
                        }
                        if filter_param:
                            new_val[FeatureFlagConstants.FILTER_PARAMETERS] = (
                                filter_param
                            )
                        feature_flag_value.conditions[
                            FeatureFlagConstants.CLIENT_FILTERS
                        ][idx] = new_val
            elif isinstance(v, bool):
                feature_flag_value.enabled = v
                feature_flag_value.conditions = default_conditions

            else:
                raise ValueError(
                    "The type of '{}' should be either boolean or dictionary.".format(v)
                )

            feature_flags.append(feature_flag_value)

    return feature_flags


def __import_kvset_from_file(
    client,
    path,
    strict,
    yes,
    dry_run=False,
    import_mode=ImportMode.IGNORE_MATCH,
    correlation_request_id=None,
):
    new_kvset = __read_with_appropriate_encoding(file_path=path, format_="json")
    if KVSetConstants.KVSETRootElementName not in new_kvset:
        raise FileOperationError(
            "file '{0}' is not in a valid '{1}' format.".format(
                path, ImportExportProfiles.KVSET
            )
        )

    kvset_from_file = [
        ConfigurationSetting(
            key=kv.get("key", None),
            label=kv.get("label", None),
            content_type=kv.get("content_type", None),
            value=kv.get("value", None),
            tags=kv.get("tags", None),
        )
        for kv in new_kvset[KVSetConstants.KVSETRootElementName]
    ]

    kvset_from_file = list(filter(__validate_import_config_setting, kvset_from_file))

    existing_kvset = __read_kv_from_config_store(
        client, key=SearchFilterOptions.ANY_KEY, label=SearchFilterOptions.ANY_LABEL
    )

    comparer = KVComparer(kvset_from_file, CompareFieldsMap["kvset"])
    diff = comparer.compare(existing_kvset, strict=strict)

    changes_detected = print_preview(
        diff,
        level="kvset",
        yes=yes,
        strict=strict,
        title="KVSet",
        indent=2,
        show_update_diff=False,
    )

    if not changes_detected or dry_run:
        return

    if not yes:
        user_confirmation("Do you want to continue?\n")

    for config_setting in diff.get(JsonDiff.DELETE, []):
        __delete_configuration_setting_from_config_store(client, config_setting)

    # Create joint iterable from added and updated kvs
    if import_mode == ImportMode.IGNORE_MATCH:
        kvset_to_import_iter = chain(
            diff.get(JsonDiff.ADD, []),
            (update["new"] for update in diff.get(JsonDiff.UPDATE, [])),
        )  # The value of diff update property is of the form List[{"new": KeyValue, "old": KeyValue}]
    else:
        kvset_to_import_iter = kvset_from_file

    for config_setting in kvset_to_import_iter:
        __write_configuration_setting_to_config_store(
            client, config_setting, correlation_request_id
        )


def __read_kv_from_file(
    file_path, format_, separator=None, prefix_to_add="", depth=None, content_type=None
):
    config_data = {}
    try:
        config_data = __read_with_appropriate_encoding(file_path, format_)
        if format_ in ("json", "yaml"):
            for feature_management_keyword in (
                keywords.feature_management
                for keywords in FeatureManagementReservedKeywords.ALL
            ):
                # delete all feature management sections in any name format.
                # If users have not skipped features, and there are multiple
                # feature sections, we will error out while reading features.
                if feature_management_keyword in config_data:
                    del config_data[feature_management_keyword]
    except ValueError as ex:
        raise FileOperationError(
            "The input is not a well formatted %s file.\nException: %s" % (format_, ex)
        )
    except yaml.YAMLError as ex:
        raise FileOperationError(
            "The input is not a well formatted YAML file.\nException: %s" % (ex)
        )
    except OSError:
        raise FileOperationError("File is not available.")

    flattened_data = __flatten_config_data(
        config_data=config_data,
        format_=format_,
        content_type=content_type,
        prefix_to_add=prefix_to_add,
        depth=depth,
        separator=separator
    )

    # convert to KeyValue list
    key_values = []
    for k, v in flattened_data.items():
        if validate_import_key(key=k):
            key_values.append(KeyValue(key=k, value=v))
    return key_values


def __flatten_config_data(config_data, format_, content_type, prefix_to_add="", depth=None, separator=None):
    """
    Flatten configuration data into a dictionary of key-value pairs.

    Args:
        config_data: The configuration data to flatten (dict or list)
        format_ (str): The format of the configuration data ('json', 'yaml', 'properties')
        content_type (str): Content type for JSON validation
        prefix_to_add (str): Prefix to add to each key
        depth (int): Maximum depth for flattening hierarchical data
        separator (str): Separator for hierarchical keys

    Returns:
        dict: Flattened key-value pairs
    """
    flattened_data = {}

    if format_ == "json" and content_type and is_json_content_type(content_type):
        for key in config_data:
            __flatten_json_key_value(
                key=prefix_to_add + key,
                value=config_data[key],
                flattened_data=flattened_data,
                depth=depth,
                separator=separator,
            )
    else:
        index = 0
        is_list = isinstance(config_data, list)
        for key in config_data:
            if is_list:
                __flatten_key_value(
                    key=prefix_to_add + str(index),
                    value=key,
                    flattened_data=flattened_data,
                    depth=depth,
                    separator=separator,
                )
                index += 1
            else:
                __flatten_key_value(
                    key=prefix_to_add + key,
                    value=config_data[key],
                    flattened_data=flattened_data,
                    depth=depth,
                    separator=separator,
                )

    return flattened_data

# App Service <-> List of KeyValue object


def __read_kv_from_app_service(
    cmd, appservice_account, prefix_to_add="", content_type=None
):
    try:
        key_values = []
        from azure.cli.command_modules.appservice.custom import get_app_settings

        slot = (
            appservice_account.get("resource_name")
            if appservice_account.get("resource_type") == "slots"
            else None
        )
        settings = get_app_settings(
            cmd,
            resource_group_name=appservice_account["resource_group"],
            name=appservice_account["name"],
            slot=slot,
        )
        for item in settings:
            key = prefix_to_add + item["name"]
            value = item["value"]

            if (
                value.strip()
                .lower()
                .startswith(AppServiceConstants.APPSVC_CONFIG_REFERENCE_PREFIX.lower())
            ):  # Exclude app configuration references.
                logger.warning(
                    'Ignoring app configuration reference with Key "%s" and Value "%s"',
                    key,
                    value,
                )
                continue

            if validate_import_key(key):
                tags = (
                    {
                        AppServiceConstants.APPSVC_SLOT_SETTING_KEY: str(
                            item["slotSetting"]
                        ).lower()
                    }
                    if item["slotSetting"]
                    else {}
                )

                # Value will look like one of the following if it is a KeyVault reference:
                # @Microsoft.KeyVault(SecretUri=https://myvault.vault.azure.net/secrets/mysecret/ec96f02080254f109c51a1f14cdb1931)
                # @Microsoft.KeyVault(VaultName=myvault;SecretName=mysecret;SecretVersion=ec96f02080254f109c51a1f14cdb1931)
                if value and value.strip().lower().startswith(
                    AppServiceConstants.APPSVC_KEYVAULT_PREFIX.lower()
                ):
                    try:
                        # Strip all whitespaces from value string.
                        # Valid values of SecretUri, VaultName, SecretName or SecretVersion will never have whitespaces.
                        value = value.replace(" ", "")
                        appsvc_value_dict = dict(
                            x.split("=")
                            for x in value[
                                len(AppServiceConstants.APPSVC_KEYVAULT_PREFIX) + 1: -1
                            ].split(";")
                        )
                        appsvc_value_dict = {
                            k.lower(): v for k, v in appsvc_value_dict.items()
                        }
                        secret_identifier = appsvc_value_dict.get("secreturi")
                        if not secret_identifier:
                            # Construct secreturi
                            vault_name = appsvc_value_dict.get("vaultname")
                            secret_name = appsvc_value_dict.get("secretname")
                            secret_version = appsvc_value_dict.get("secretversion")
                            secret_identifier = (
                                "https://{0}.vault.azure.net/secrets/{1}/{2}".format(
                                    vault_name, secret_name, secret_version
                                )
                            )
                        try:
                            # this throws an exception for invalid format of secret identifier
                            parse_key_vault_id(source_id=secret_identifier)
                            kv = KeyValue(
                                key=key,
                                value=json.dumps(
                                    {"uri": secret_identifier}, ensure_ascii=False
                                ),
                                tags=tags,
                                content_type=KeyVaultConstants.KEYVAULT_CONTENT_TYPE,
                            )
                            key_values.append(kv)
                            continue
                        except (TypeError, ValueError) as e:
                            logger.debug(
                                'Exception while validating the format of KeyVault identifier. Key "%s" with value "%s" will be treated like a regular key-value.\n%s',
                                key,
                                value,
                                str(e),
                            )
                    except (AttributeError, TypeError, ValueError) as e:
                        logger.debug(
                            'Key "%s" with value "%s" is not a well-formatted KeyVault reference. It will be treated like a regular key-value.\n%s',
                            key,
                            value,
                            str(e),
                        )

                elif content_type and is_json_content_type(content_type):
                    # If appservice values are being imported with JSON content type,
                    # we need to validate that values are in valid JSON format.
                    try:
                        json.loads(value)
                    except ValueError:
                        raise ValidationError(
                            'Value "{}" for key "{}" is not a valid JSON object, which conflicts with the provided content type "{}".'.format(
                                value, key, content_type
                            )
                        )

                kv = KeyValue(key=key, value=value, tags=tags)
                key_values.append(kv)
        return key_values
    except Exception as exception:
        raise CLIError("Failed to read key-values from appservice.\n" + str(exception))


def __read_kv_from_kubernetes_configmap(
    cmd,
    aks_cluster,
    configmap_name,
    format_,
    namespace="default",
    prefix_to_add="",
    content_type=None,
    depth=None,
    separator=None
):
    """
    Read key-value pairs from a Kubernetes ConfigMap using aks_runcommand.

    Args:
        cmd: The command context object
        aks_cluster (str): Name of the AKS cluster
        configmap_name (str): Name of the ConfigMap to read from
        format_ (str): Format of the data in the ConfigMap (e.g., "json", "yaml")
        namespace (str): Kubernetes namespace where the ConfigMap resides (default: "default")
        prefix_to_add (str): Prefix to add to each key in the ConfigMap
        content_type (str): Content type to apply to the key-values
        depth (int): Maximum depth for flattening hierarchical data
        separator (str): Separator for hierarchical keys

    Returns:
        list: List of KeyValue objects
    """
    key_values = []
    from azure.cli.command_modules.acs.custom import aks_runcommand
    from azure.cli.command_modules.acs._client_factory import cf_managed_clusters

    # Preserve only the necessary CLI context data
    original_subscription = cmd.cli_ctx.data.get('subscription_id')
    original_safe_params = cmd.cli_ctx.data.get('safe_params', [])

    try:
        # Temporarily modify the CLI context
        cmd.cli_ctx.data['subscription_id'] = aks_cluster["subscription"]
        params_to_keep = ["--debug", "--verbose"]
        cmd.cli_ctx.data['safe_params'] = [p for p in original_safe_params if p in params_to_keep]
        # It must be set to return the result.
        cmd.cli_ctx.data['safe_params'].append("--output")
        # Get the AKS client from the factory
        aks_client = cf_managed_clusters(cmd.cli_ctx)

        # Command to get the ConfigMap and output it as JSON
        command = f"kubectl get configmap {configmap_name} -n {namespace} -o json"

        # Execute the command on the cluster
        result = aks_runcommand(cmd, aks_client, aks_cluster["resource_group"], aks_cluster["name"], command_string=command)

        if hasattr(result, 'logs') and result.logs:
            if not hasattr(result, 'exit_code') or result.exit_code == 0:
                try:
                    configmap_data = json.loads(result.logs)

                    # Extract the data section which contains the key-value pairs
                    kvs = __extract_kv_from_configmap_data(
                        configmap_data, content_type, prefix_to_add, format_, depth, separator)

                    key_values.extend(kvs)
                except json.JSONDecodeError:
                    raise ValueError(
                        f"The result from ConfigMap {configmap_name} could not be parsed. {result.logs.strip()}"
                    )
            else:
                raise AzureResponseError(f"{result.logs.strip()}")
        else:
            raise AzureResponseError("Unable to get the ConfigMap.")

        return key_values
    except Exception as exception:
        raise AzureInternalError(
            f"Failed to read key-values from ConfigMap '{configmap_name}' in namespace '{namespace}'.\n{str(exception)}"
        )
    finally:
        # Restore original CLI context data
        cmd.cli_ctx.data['subscription_id'] = original_subscription
        cmd.cli_ctx.data['safe_params'] = original_safe_params


def __extract_kv_from_configmap_data(configmap, content_type, prefix_to_add="", format_=None, depth=None, separator=None):
    """
    Helper function to extract key-value pairs from ConfigMap data.

    Args:
        configmap (dict): The ConfigMap data as a dictionary
        prefix_to_add (str): Prefix to add to each key
        content_type (str): Content type to apply to the key-values
        format_ (str): Format of the data in the ConfigMap (e.g., "json", "yaml")
        depth (int): Maximum depth for flattening hierarchical data
        separator (str): Separator for hierarchical keys

    Returns:
        list: List of KeyValue objects
    """
    key_values = []

    if not configmap.get('data', None):
        logger.warning("ConfigMap exists but has no data")
        return key_values

    for key, value in configmap['data'].items():
        if format_ in ("json", "yaml", "properties"):
            if format_ == "json":
                try:
                    value = json.loads(value)
                except json.JSONDecodeError:
                    logger.warning(
                        'Value "%s" for key "%s" is not a well formatted JSON data.',
                        value, key
                    )
                    continue
            elif format_ == "yaml":
                try:
                    value = yaml.safe_load(value)
                except yaml.YAMLError:
                    logger.warning(
                        'Value "%s" for key "%s" is not a well formatted YAML data.',
                        value, key
                    )
                    continue
            else:
                try:
                    value = javaproperties.load(io.StringIO(value))
                except javaproperties.InvalidUEscapeError:
                    logger.warning(
                        'Value "%s" for key "%s" is not a well formatted properties data.',
                        value, key
                    )
                    continue

            flattened_data = __flatten_config_data(
                config_data=value,
                format_=format_,
                content_type=content_type,
                prefix_to_add=prefix_to_add,
                depth=depth,
                separator=separator
            )

            for k, v in flattened_data.items():
                if validate_import_key(key=k):
                    key_values.append(KeyValue(key=k, value=v))

        elif validate_import_key(key):
            # If content_type is JSON, validate the value
            if content_type and is_json_content_type(content_type):
                try:
                    json.loads(value)
                except json.JSONDecodeError:
                    logger.warning(
                        'Value "%s" for key "%s" is not a valid JSON object, which conflicts with the provided content type "%s".',
                        value, key, content_type
                    )
                    continue

            kv = KeyValue(key=prefix_to_add + key, value=value)
            key_values.append(kv)

    return key_values


def __validate_import_keyvault_ref(kv):
    if kv and validate_import_key(kv.key):
        try:
            value = json.loads(kv.value)
        except JSONDecodeError as exception:
            logger.warning(
                "The keyvault reference with key '{%s}' is not in a valid JSON format. It will not be imported.\n{%s}",
                kv.key,
                str(exception),
            )
            return False

        if "uri" in value:
            parsed_url = urlparse(value["uri"])
            # URL with a valid scheme and netloc is a valid url, but keyvault ref has path as well, so validate it
            if parsed_url.scheme and parsed_url.netloc and parsed_url.path:
                try:
                    parse_key_vault_id(source_id=value["uri"])
                    return True
                except Exception:  # pylint: disable=broad-except
                    pass

        logger.warning(
            "Keyvault reference with key '{%s}' is not a valid keyvault reference. It will not be imported.",
            kv.key,
        )
    return False


def __validate_import_feature_flag(kv):
    if kv and validate_import_feature_key(kv.key):
        try:
            ff = json.loads(kv.value)
            if FEATURE_FLAG_PROPERTIES.union(ff.keys()) == FEATURE_FLAG_PROPERTIES:
                return validate_import_feature(ff[FeatureFlagConstants.ID])

            logger.warning(
                "The feature flag with key '%s' is not a valid feature flag. It will not be imported.",
                kv.key,
            )
        except JSONDecodeError as exception:
            logger.warning(
                "The feature flag with key '%s' is not in a valid JSON format. It will not be imported.\n%s",
                kv.key,
                str(exception),
            )
    return False


def __validate_import_config_setting(config_setting):
    if __is_key_vault_ref(kv=config_setting):
        if not __validate_import_keyvault_ref(kv=config_setting):
            return False
    elif is_feature_flag(kv=config_setting):
        if not __validate_import_feature_flag(kv=config_setting):
            return False
    elif not validate_import_key(config_setting.key):
        return False

    if config_setting.value and not isinstance(config_setting.value, str):
        logger.warning(
            "The 'value' for the key '{%s}' is not a string. This key-value will not be imported.",
            config_setting.key,
        )
        return False
    if config_setting.content_type and not isinstance(config_setting.content_type, str):
        logger.warning(
            "The 'content_type' for the key '{%s}' is not a string. This key-value will not be imported.",
            config_setting.key,
        )
        return False
    if config_setting.label and not isinstance(config_setting.label, str):
        logger.warning(
            "The 'label' for the key '{%s}' is not a string. This key-value will not be imported.",
            config_setting.key,
        )
        return False

    return __validate_import_tags(config_setting)


def __validate_import_tags(kv):
    if kv.tags and not isinstance(kv.tags, dict):
        logger.warning(
            "The format of 'tags' for key '%s' is not valid. This key-value will not be imported.",
            kv.key,
        )
        return False

    if kv.tags:
        for tag_key, tag_value in kv.tags.items():
            if not isinstance(tag_value, str):
                logger.warning(
                    "The value for the tag '{%s}' for key '{%s}' is not in a valid format. This key-value will not be imported.",
                    tag_key,
                    kv.key,
                )
                return False
    return True


def validate_import_key(key):
    if key:
        if not isinstance(key, str):
            logger.warning("Ignoring invalid key '%s'. Key must be a string.", key)
            return False
        if key == "." or key == ".." or "%" in key:
            logger.warning(
                "Ignoring invalid key '%s'. Key cannot be a '.' or '..', or contain the '%%' character.",
                key,
            )
            return False
        if key.startswith(FeatureFlagConstants.FEATURE_FLAG_PREFIX):
            logger.warning(
                "Ignoring invalid key '%s'. Key cannot start with the reserved prefix for feature flags.",
                key,
            )
            return False
    else:
        logger.warning("Ignoring invalid key ''. Key cannot be empty.")
        return False
    return True


def validate_import_feature(feature):
    try:
        validate_feature_flag_name(feature)
    except InvalidArgumentValueError as exception:
        logger.warning(
            "Ignoring invalid feature '%s'. %s", feature, exception.error_msg
        )
        return False

    return True


def validate_import_feature_key(key):
    try:
        validate_feature_flag_key(key)
    except InvalidArgumentValueError as exception:
        logger.warning(
            "Ignoring invalid feature with key '%s'. %s", key, exception.error_msg
        )
        return False

    return True


def __check_file_encoding(file_path):
    with open(file_path, "rb") as config_file:
        data = config_file.readline()
        encoding_type = chardet.detect(data)["encoding"]
        return encoding_type


def __delete_configuration_setting_from_config_store(
    azconfig_client, configuration_setting
):
    try:
        azconfig_client.delete_configuration_setting(
            key=configuration_setting.key, label=configuration_setting.label
        )
    except ResourceReadOnlyError:
        logger.warning(
            "Failed to delete read only key-value with key '%s' and label '%s'. Unlock the key-value before deleting it.",
            configuration_setting.key,
            configuration_setting.label,
        )
    except HttpResponseError as exception:
        logger.warning(
            "Failed to delete key-value with key '%s' and label '%s'. %s",
            configuration_setting.key,
            configuration_setting.label,
            str(exception),
        )
    except Exception as exception:
        raise AzureInternalError(str(exception))


def __flatten_key_value(key, value, flattened_data, depth, separator):
    if depth > 1:
        depth = depth - 1
        if isinstance(value, list):
            if separator is None or not separator:
                raise RequiredArgumentMissingError(
                    "A non-empty separator is required for importing hierarchical configurations."
                )
            for index, item in enumerate(value):
                __flatten_key_value(
                    key + separator + str(index), item, flattened_data, depth, separator
                )
        elif isinstance(value, dict):
            if separator is None or not separator:
                raise RequiredArgumentMissingError(
                    "A non-empty separator is required for importing hierarchical configurations."
                )
            for nested_key in value:
                __flatten_key_value(
                    key + separator + nested_key,
                    value[nested_key],
                    flattened_data,
                    depth,
                    separator,
                )
        else:
            if key in flattened_data:
                logger.debug(
                    "The key %s already exist, value has been overwritten.", key
                )
            flattened_data[key] = (
                value if isinstance(value, str) else json.dumps(value)
            )  # Ensure boolean values are properly stringified.
    else:
        flattened_data[key] = value if isinstance(value, str) else json.dumps(value)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
# pylint: disable=too-many-statements
# pylint: disable=too-many-locals

from azure.cli.core.commands import CliCommandType


def load_command_table(self, _):
    eh_namespace_custom = CliCommandType(
        operations_tmpl='azure.cli.command_modules.eventhubs.operations.namespace_custom#{}',
    )

    eh_appgroup_custom = CliCommandType(
        operations_tmpl='azure.cli.command_modules.eventhubs.operations.app_group_custom_file#{}'
    )

    eh_eventhub_custom = CliCommandType(
        operations_tmpl='azure.cli.command_modules.eventhubs.operations.event_hub_entity#{}',
    )

    eh_network_custom = CliCommandType(
        operations_tmpl='azure.cli.command_modules.eventhubs.operations.network_rule_set#{}'
    )

# Namespace Region
    with self.command_group('eventhubs namespace', custom_command_type=eh_namespace_custom) as g:
        g.custom_command('create', 'create_eventhub_namespace')

    with self.command_group('eventhubs namespace private-endpoint-connection', custom_command_type=eh_namespace_custom) as g:
        from ._validator import validate_private_endpoint_connection_id
        g.custom_command('approve', 'approve_private_endpoint_connection', validator=validate_private_endpoint_connection_id)
        g.custom_command('reject', 'reject_private_endpoint_connection', validator=validate_private_endpoint_connection_id)
        g.custom_command('delete', 'delete_private_endpoint_connection', confirmation=True, validator=validate_private_endpoint_connection_id)

# EventHub Region
    from .operations.event_hub_entity import EventHubEntityUpdate
    self.command_table['eventhubs eventhub update'] = EventHubEntityUpdate(loader=self)
    with self.command_group('eventhubs eventhub', custom_command_type=eh_eventhub_custom) as g:
        g.custom_command('create', 'cli_eventhub_create')

# DisasterRecoveryConfigs Region
    with self.command_group('eventhubs georecovery-alias', custom_command_type=eh_namespace_custom) as g:
        g.custom_command('set', 'set_georecovery_alias', supports_no_wait=True)

# NetworkRuleSet Region
    with self.command_group('eventhubs namespace network-rule-set ip-rule', custom_command_type=eh_network_custom) as g:
        g.custom_command('add', 'add_network_rule_set_ip_rule')
        g.custom_command('remove', 'remove_network_rule_set_ip_rule')

    with self.command_group('eventhubs namespace network-rule-set virtual-network-rule', custom_command_type=eh_network_custom) as g:
        g.custom_command('add', 'add_virtual_network_rule')
        g.custom_command('remove', 'remove_virtual_network_rule')

# Identity Region
    with self.command_group('eventhubs namespace identity', custom_command_type=eh_namespace_custom) as g:
        g.custom_command('assign', 'cli_add_identity')
        g.custom_command('remove', 'cli_remove_identity')

# Encryption Region
    with self.command_group('eventhubs namespace encryption', custom_command_type=eh_namespace_custom) as g:
        g.custom_command('add', 'cli_add_encryption')
        g.custom_command('remove', 'cli_remove_encryption')

# ApplicationGroup Region
    with self.command_group('eventhubs namespace application-group', custom_command_type=eh_appgroup_custom) as g:
        g.custom_command('create', 'cli_appgroup_create')

    with self.command_group('eventhubs namespace application-group policy', custom_command_type=eh_appgroup_custom) as g:
        g.custom_command('add', 'cli_add_appgroup_policy')
        g.custom_command('remove', 'cli_remove_appgroup_policy')
# Replica Location Region
    with self.command_group('eventhubs namespace replica', custom_command_type=eh_namespace_custom) as g:
        g.custom_command('add', 'cli_add_location')
        g.custom_command('remove', 'cli_remove_location')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "managedservices assignment create",
)
class Create(AAZCommand):
    """Create a registration assignment.

    :example: Create an assignment under the default subscription scope.
        az managedservices assignment create --definition "/subscriptions/a62076fa-768a-403c-9d9d-6a9919aae441/providers/Microsoft.ManagedServices/registrationDefinitions/0c3e9687-b461-4615-b6e4-74d54998d6e4"

    :example: Create an assignment under a given resource group scope.
        az managedservices assignment create --definition "/subscriptions/a62076fa-768a-403c-9d9d-6a9919aae441/providers/Microsoft.ManagedServices/registrationDefinitions/0c3e9687-b461-4615-b6e4-74d54998d6e4" --resource-group mygroup
    """

    _aaz_info = {
        "version": "2019-06-01",
        "resources": [
            ["mgmt-plane", "/{scope}/providers/microsoft.managedservices/registrationassignments/{}", "2019-06-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.assignment_id = AAZStrArg(
            options=["--assignment-id"],
            help="Can be used to override the generated registration assignment id.",
            required=True,
        )
        _args_schema.scope = AAZStrArg(
            options=["--scope"],
            help="Scope of the resource.",
            required=True,
        )
        _args_schema.definition = AAZStrArg(
            options=["--definition"],
            help="Fully qualified path of the registration definition.",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.RegistrationAssignmentsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class RegistrationAssignmentsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/{scope}/providers/Microsoft.ManagedServices/registrationAssignments/{registrationAssignmentId}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "registrationAssignmentId", self.ctx.args.assignment_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "scope", self.ctx.args.scope,
                    skip_quote=True,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2019-06-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("properties", AAZObjectType)

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("registrationDefinitionId", AAZStrType, ".definition", typ_kwargs={"flags": {"required": True}})

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.properties = AAZObjectType()
            _schema_on_200_201.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200_201.properties
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.registration_definition = AAZObjectType(
                serialized_name="registrationDefinition",
                flags={"read_only": True},
            )
            properties.registration_definition_id = AAZStrType(
                serialized_name="registrationDefinitionId",
                flags={"required": True},
            )

            registration_definition = cls._schema_on_200_201.properties.registration_definition
            registration_definition.id = AAZStrType(
                flags={"read_only": True},
            )
            registration_definition.name = AAZStrType(
                flags={"read_only": True},
            )
            registration_definition.plan = AAZObjectType()
            registration_definition.properties = AAZObjectType()
            registration_definition.type = AAZStrType(
                flags={"read_only": True},
            )

            plan = cls._schema_on_200_201.properties.registration_definition.plan
            plan.name = AAZStrType(
                flags={"required": True},
            )
            plan.product = AAZStrType(
                flags={"required": True},
            )
            plan.publisher = AAZStrType(
                flags={"required": True},
            )
            plan.version = AAZStrType(
                flags={"required": True},
            )

            properties = cls._schema_on_200_201.properties.registration_definition.properties
            properties.authorizations = AAZListType()
            properties.description = AAZStrType()
            properties.managed_by_tenant_id = AAZStrType(
                serialized_name="managedByTenantId",
            )
            properties.managed_by_tenant_name = AAZStrType(
                serialized_name="managedByTenantName",
            )
            properties.managee_tenant_id = AAZStrType(
                serialized_name="manageeTenantId",
            )
            properties.managee_tenant_name = AAZStrType(
                serialized_name="manageeTenantName",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
            )
            properties.registration_definition_name = AAZStrType(
                serialized_name="registrationDefinitionName",
            )

            authorizations = cls._schema_on_200_201.properties.registration_definition.properties.authorizations
            authorizations.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.registration_definition.properties.authorizations.Element
            _element.delegated_role_definition_ids = AAZListType(
                serialized_name="delegatedRoleDefinitionIds",
            )
            _element.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"required": True},
            )
            _element.principal_id_display_name = AAZStrType(
                serialized_name="principalIdDisplayName",
            )
            _element.role_definition_id = AAZStrType(
                serialized_name="roleDefinitionId",
                flags={"required": True},
            )

            delegated_role_definition_ids = cls._schema_on_200_201.properties.registration_definition.properties.authorizations.Element.delegated_role_definition_ids
            delegated_role_definition_ids.Element = AAZStrType()

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""


__all__ = ["Create"]



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=too-many-lines

from knack.help_files import helps


helps['sshkey'] = """
    type: group
    short-summary: Manage ssh public key with vm
"""

helps['sshkey list'] = """
    type: command
    short-summary: "List all of the SSH public keys."
"""

helps['sshkey show'] = """
    type: command
    short-summary: "Retrieve information about an SSH public key."
    examples:
      - name: Get an ssh public key.
        text: |-
               az sshkey show --resource-group "myResourceGroup" --name "mySshPublicKeyName"
"""

helps['sshkey create'] = """
    type: command
    short-summary: "Create a new SSH public key resource."
    examples:
      - name: Create a new SSH public key resource.
        text: |-
               az sshkey create --location "westus" --public-key "{ssh-rsa public key}" --resource-group \
"myResourceGroup" --name "mySshPublicKeyName"
      - name: Create a new SSH public key resource using public key in a file.
        text: |-
               az sshkey create --location "westus" --public-key "@filename" --resource-group \
"myResourceGroup" --name "mySshPublicKeyName"
      - name: Create a new SSH public key resource with auto-generated value.
        text: |-
               az sshkey create --location "westus" --resource-group "myResourceGroup" --name "mySshPublicKeyName"
      - name: Create a new SSH public key resource with Ed25519 encryption.
        text: |-
               az sshkey create --location "westus" --resource-group "myResourceGroup" --name "mySshPublicKeyName" \
--encryption-type "Ed25519"
"""

helps['sshkey update'] = """
    type: command
    short-summary: "Update an SSH public key resource."
"""

helps['sshkey delete'] = """
    type: command
    short-summary: "Delete an SSH public key."
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "netappfiles account show",
)
class Show(AAZCommand):
    """Get the NetApp account

    :example: Get an ANF account
        az netappfiles account show -g mygroup --name myname
    """

    _aaz_info = {
        "version": "2025-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.netapp/netappaccounts/{}", "2025-01-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.account_name = AAZStrArg(
            options=["-a", "-n", "--name", "--account-name"],
            help="The name of the NetApp account",
            required=True,
            id_part="name",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z0-9][a-zA-Z0-9\\-_]{0,127}$",
            ),
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.AccountsGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class AccountsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.NetApp/netAppAccounts/{accountName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "accountName", self.ctx.args.account_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.identity = AAZObjectType()
            _schema_on_200.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            identity = cls._schema_on_200.identity
            identity.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )
            identity.tenant_id = AAZStrType(
                serialized_name="tenantId",
                flags={"read_only": True},
            )
            identity.type = AAZStrType(
                flags={"required": True},
            )
            identity.user_assigned_identities = AAZDictType(
                serialized_name="userAssignedIdentities",
            )

            user_assigned_identities = cls._schema_on_200.identity.user_assigned_identities
            user_assigned_identities.Element = AAZObjectType(
                nullable=True,
            )

            _element = cls._schema_on_200.identity.user_assigned_identities.Element
            _element.client_id = AAZStrType(
                serialized_name="clientId",
                flags={"read_only": True},
            )
            _element.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.active_directories = AAZListType(
                serialized_name="activeDirectories",
            )
            properties.disable_showmount = AAZBoolType(
                serialized_name="disableShowmount",
                nullable=True,
                flags={"read_only": True},
            )
            properties.encryption = AAZObjectType()
            properties.multi_ad_status = AAZStrType(
                serialized_name="multiAdStatus",
                flags={"read_only": True},
            )
            properties.nfs_v4_id_domain = AAZStrType(
                serialized_name="nfsV4IDDomain",
                nullable=True,
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )

            active_directories = cls._schema_on_200.properties.active_directories
            active_directories.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.active_directories.Element
            _element.active_directory_id = AAZStrType(
                serialized_name="activeDirectoryId",
                nullable=True,
            )
            _element.ad_name = AAZStrType(
                serialized_name="adName",
            )
            _element.administrators = AAZListType()
            _element.aes_encryption = AAZBoolType(
                serialized_name="aesEncryption",
            )
            _element.allow_local_nfs_users_with_ldap = AAZBoolType(
                serialized_name="allowLocalNfsUsersWithLdap",
            )
            _element.backup_operators = AAZListType(
                serialized_name="backupOperators",
            )
            _element.dns = AAZStrType()
            _element.domain = AAZStrType()
            _element.encrypt_dc_connections = AAZBoolType(
                serialized_name="encryptDCConnections",
            )
            _element.kdc_ip = AAZStrType(
                serialized_name="kdcIP",
            )
            _element.ldap_over_tls = AAZBoolType(
                serialized_name="ldapOverTLS",
            )
            _element.ldap_search_scope = AAZObjectType(
                serialized_name="ldapSearchScope",
            )
            _element.ldap_signing = AAZBoolType(
                serialized_name="ldapSigning",
            )
            _element.organizational_unit = AAZStrType(
                serialized_name="organizationalUnit",
            )
            _element.password = AAZStrType(
                flags={"secret": True},
            )
            _element.preferred_servers_for_ldap_client = AAZStrType(
                serialized_name="preferredServersForLdapClient",
            )
            _element.security_operators = AAZListType(
                serialized_name="securityOperators",
            )
            _element.server_root_ca_certificate = AAZStrType(
                serialized_name="serverRootCACertificate",
                flags={"secret": True},
            )
            _element.site = AAZStrType()
            _element.smb_server_name = AAZStrType(
                serialized_name="smbServerName",
            )
            _element.status = AAZStrType(
                flags={"read_only": True},
            )
            _element.status_details = AAZStrType(
                serialized_name="statusDetails",
                flags={"read_only": True},
            )
            _element.username = AAZStrType()

            administrators = cls._schema_on_200.properties.active_directories.Element.administrators
            administrators.Element = AAZStrType()

            backup_operators = cls._schema_on_200.properties.active_directories.Element.backup_operators
            backup_operators.Element = AAZStrType()

            ldap_search_scope = cls._schema_on_200.properties.active_directories.Element.ldap_search_scope
            ldap_search_scope.group_dn = AAZStrType(
                serialized_name="groupDN",
            )
            ldap_search_scope.group_membership_filter = AAZStrType(
                serialized_name="groupMembershipFilter",
            )
            ldap_search_scope.user_dn = AAZStrType(
                serialized_name="userDN",
            )

            security_operators = cls._schema_on_200.properties.active_directories.Element.security_operators
            security_operators.Element = AAZStrType()

            encryption = cls._schema_on_200.properties.encryption
            encryption.identity = AAZObjectType()
            encryption.key_source = AAZStrType(
                serialized_name="keySource",
            )
            encryption.key_vault_properties = AAZObjectType(
                serialized_name="keyVaultProperties",
            )

            identity = cls._schema_on_200.properties.encryption.identity
            identity.federated_client_id = AAZStrType(
                serialized_name="federatedClientId",
            )
            identity.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )
            identity.user_assigned_identity = AAZStrType(
                serialized_name="userAssignedIdentity",
            )

            key_vault_properties = cls._schema_on_200.properties.encryption.key_vault_properties
            key_vault_properties.key_name = AAZStrType(
                serialized_name="keyName",
                flags={"required": True},
            )
            key_vault_properties.key_vault_id = AAZStrType(
                serialized_name="keyVaultId",
                flags={"read_only": True},
            )
            key_vault_properties.key_vault_resource_id = AAZStrType(
                serialized_name="keyVaultResourceId",
            )
            key_vault_properties.key_vault_uri = AAZStrType(
                serialized_name="keyVaultUri",
                flags={"required": True},
            )
            key_vault_properties.status = AAZStrType(
                flags={"read_only": True},
            )

            system_data = cls._schema_on_200.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200


class _ShowHelper:
    """Helper class for Show"""


__all__ = ["Show"]
